package cn.wise.sc.consume.business.security;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.consume.business.model.LoginUser;
import cn.wise.sc.consume.business.service.TokenService;
import cn.wise.sc.consume.business.util.JwtUtil;
import cn.wise.sc.consume.business.util.UserUtil;
import com.auth0.jwt.JWT;
import com.auth0.jwt.exceptions.JWTDecodeException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.security.web.util.matcher.RequestHeaderRequestMatcher;
import org.springframework.security.web.util.matcher.RequestMatcher;
import org.springframework.util.Assert;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class JwtAuthenticationFilter extends OncePerRequestFilter {

    private RequestMatcher requiresAuthenticationRequestMatcher;
    private List<RequestMatcher> permissiveRequestMatchers;
    private AuthenticationManager authenticationManager;
	@Autowired
	private TokenService tokenService;
	@Autowired
	private UserDetailsService userDetailsService;
	private static final Long MINUTES_10 = 10 * 60 * 1000L;
	public static final String TOKEN_KEY = "token";


    private AuthenticationSuccessHandler successHandler = new SavedRequestAwareAuthenticationSuccessHandler();
    private AuthenticationFailureHandler failureHandler = new HttpStatusLoginFailureHandler();

    public JwtAuthenticationFilter() {
        this.requiresAuthenticationRequestMatcher = new RequestHeaderRequestMatcher("Authorization");
    }

    @Override
    public void afterPropertiesSet() {
        Assert.notNull(authenticationManager, "authenticationManager must be specified");
        Assert.notNull(successHandler, "AuthenticationSuccessHandler must be specified");
        Assert.notNull(failureHandler, "AuthenticationFailureHandler must be specified");
    }

    protected String getJwtToken(HttpServletRequest request) {
        String authInfo = request.getHeader("Authorization");
        return StringUtils.removeStart(authInfo, "Bearer ");
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
	    String pf = request.getHeader("platform");
	    if(StringUtils.isEmpty(pf)){
		    if (!requiresAuthentication(request, response)) {
			    filterChain.doFilter(request, response);
			    return;
		    }
		    Authentication authResult = null;
		    AuthenticationException failed = null;
		    try {
			    String token = getJwtToken(request);
			    if (StringUtils.isNotBlank(token)) {
				    JwtAuthenticationToken authToken = new JwtAuthenticationToken(JWT.decode(token));
				    authResult = this.getAuthenticationManager().authenticate(authToken);
			    } else {
				    failed = new InsufficientAuthenticationException("JWT is Empty");
			    }
		    } catch (JWTDecodeException e) {
			    logger.error("JWT format error", e);
			    failed = new InsufficientAuthenticationException("JWT format error", failed);
		    } catch (InternalAuthenticationServiceException e) {
			    logger.error(
					    "An internal error occurred while trying to authenticate the user.",
					    failed);
			    failed = e;
		    } catch (AuthenticationException e) {
			    // Authentication failed
			    failed = e;
		    }
		    if (authResult != null) {
			    successfulAuthentication(request, response, filterChain, authResult);
		    } else if (!permissiveRequest(request)) {
			    unsuccessfulAuthentication(request, response, failed);
			    return;
		    }
		    setHeaderSubTenant(request);
	    }else{
		    String token = request.getHeader("Authorization");
		    Authentication authResult = null;
		    AuthenticationException failed = null;
		    if (StringUtils.isNotBlank(token)) {
			    LoginUser loginUser = null;
			    try {
				    loginUser = JwtUtil.parserToken(token);
			    } catch (Exception e) {
				    e.printStackTrace();
			    }
			    if (loginUser != null) {
				    loginUser = checkLoginTime(loginUser);
				    UsernamePasswordAuthenticationToken authentication = new UsernamePasswordAuthenticationToken(loginUser,
						    null,null);
				    SecurityContextHolder.getContext().setAuthentication(authentication);
			    }
//			    LoginUser loginUser = tokenService.getLoginUser(token);
		    }

	    }
	    //判断添加subTenantId
        filterChain.doFilter(request, response);
    }

	/**
	 * 校验时间<br>
	 * 过期时间与当前时间对比，临近过期10分钟内的话，自动刷新缓存
	 *
	 * @param loginUser
	 * @return
	 */
	private LoginUser checkLoginTime(LoginUser loginUser) {
		long expireTime = loginUser.getExpireTime();
		long currentTime = System.currentTimeMillis();
		if (expireTime - currentTime <= MINUTES_10) {
			String token = loginUser.getToken();
			loginUser = (LoginUser) userDetailsService.loadUserByUsername(loginUser.getUsername()+"&app");
			loginUser.setToken(token);
			tokenService.refresh(loginUser);
		}
		return loginUser;
	}

	/**
	 * 如果从请求头中获取到了Sub-Tenant-Id标签 说明是超级管理员访问租户
	 *
	 * @param request
	 */
	private void setHeaderSubTenant(HttpServletRequest request) {
		String authInfo = request.getHeader("Sub-Tenant-Id");
		if (StrUtil.isNotEmpty(authInfo)) {
			//向SecurityContextHolder中的用户信息,添加SubTenantId
			LoginUser loginUser = UserUtil.getLoginUser();
			loginUser.setSubTenantId(authInfo);
			UsernamePasswordAuthenticationToken usernamePasswordAuthenticationToken = new UsernamePasswordAuthenticationToken(
					loginUser, null, loginUser.getAuthorities());
			usernamePasswordAuthenticationToken
					.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
			SecurityContextHolder.getContext().setAuthentication(usernamePasswordAuthenticationToken);
		}
	}


    protected void unsuccessfulAuthentication(HttpServletRequest request,
                                              HttpServletResponse response, AuthenticationException failed)
            throws IOException, ServletException {
        SecurityContextHolder.clearContext();
        failureHandler.onAuthenticationFailure(request, response, failed);
    }

    protected void successfulAuthentication(HttpServletRequest request,
                                            HttpServletResponse response, FilterChain chain, Authentication authResult)
            throws IOException, ServletException {
        SecurityContextHolder.getContext().setAuthentication(authResult);
        successHandler.onAuthenticationSuccess(request, response, authResult);
    }

    protected AuthenticationManager getAuthenticationManager() {
        return authenticationManager;
    }

    public void setAuthenticationManager(AuthenticationManager authenticationManager) {
        this.authenticationManager = authenticationManager;
    }

    protected boolean requiresAuthentication(HttpServletRequest request,
                                             HttpServletResponse response) {
        return requiresAuthenticationRequestMatcher.matches(request);
    }

    protected boolean permissiveRequest(HttpServletRequest request) {
        if (permissiveRequestMatchers == null)
            return false;
        for (RequestMatcher permissiveMatcher : permissiveRequestMatchers) {
            if (permissiveMatcher.matches(request))
                return true;
        }
        return false;
    }

    public void setPermissiveUrl(String... urls) {
        if (permissiveRequestMatchers == null)
            permissiveRequestMatchers = new ArrayList<>();
        for (String url : urls)
            permissiveRequestMatchers.add(new AntPathRequestMatcher(url));
    }

    public void setAuthenticationSuccessHandler(
            AuthenticationSuccessHandler successHandler) {
        Assert.notNull(successHandler, "successHandler cannot be null");
        this.successHandler = successHandler;
    }

    public void setAuthenticationFailureHandler(
            AuthenticationFailureHandler failureHandler) {
        Assert.notNull(failureHandler, "failureHandler cannot be null");
        this.failureHandler = failureHandler;
    }

    protected AuthenticationSuccessHandler getSuccessHandler() {
        return successHandler;
    }

    protected AuthenticationFailureHandler getFailureHandler() {
        return failureHandler;
    }

}
